@@ -3,7 +3,7 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Find the top repeated note phrases of MIDI files
 
-our $VERSION = '0.1106';
+our $VERSION = '0.1200';
 
 use Moo;
 use strictures 2;
@@ -11,9 +11,8 @@ use namespace::clean;
 
 use Carp;
 use Lingua::EN::Ngram;
-use List::Util qw( shuffle );
+use List::Util qw( shuffle uniq );
 use List::Util::WeightedChoice qw( choose_weighted );
-use MIDI::Simple;
 use MIDI::Util;
 use Music::Gestalt;
 use Music::Note;
@@ -50,7 +49,7 @@ has bpm => (
 has durations => (
     is      => 'ro',
     isa     => \&_is_list,
-    default => sub { [qw( hn qn en )] },
+    default => sub { [] },
 );
 
 
@@ -143,6 +142,19 @@ has notes => (
 );
 
 
+has dura => (
+    is       => 'ro',
+    init_arg => undef,
+    default  => sub { {} },
+);
+
+has _dura_list => (
+    is       => 'ro',
+    init_arg => undef,
+    default  => sub { {} },
+);
+
+
 has net => (
     is       => 'ro',
     init_arg => undef,
@@ -159,21 +171,33 @@ sub process {
         # Counter for the tracks seen
         my $i = 0;
 
+        # Declare a tempo in microseconds
+        my $tempo = 500_000; # 120 quarter notes per minute default
+
         my $opus = MIDI::Opus->new({ from_file => $file });
 
         $analysis .= "Ngram analysis of $file:\n\tN\tReps\tPhrase\n";
 
         # Handle each track...
         for my $t ( $opus->tracks ) {
-            # Collect the note events for each track
-            my @events = grep {
-                $_->[0] eq 'note_on'    # Only consider note_on events
-                && $_->[2] != 9         # Avoid the drum channel
-                && $_->[4] != 0         # Ignore events of velocity 0
-            } $t->events;
+            my $score_r = MIDI::Score::events_r_to_score_r( $t->events_r );
+            #MIDI::Score::dump_score($score_r);
 
-            # XXX Assume that there is one channel per track
-            my $track_channel = $self->one_channel ? 0 : $events[0][2];
+            # Get the tune tempo if set
+            for my $event (@$score_r) {
+                if ($event->[0] eq 'set_tempo' && $event->[2]) {
+                    $tempo = $event->[2];
+                }
+            }
+
+            # Collect the note events for each note event
+            my @events = grep {
+                $_->[0] eq 'note'   # ['note', <start>, <duration>, <channel>, <note>, <velocity>]
+                && $_->[3] != 9     # Avoid the drum channel
+            } @$score_r;
+
+            # XXX Assume that there is only one channel per track :\
+            my $track_channel = $self->one_channel ? 0 : $events[0][3];
 
             # Skip if there are no events and no channel
             next unless @events && defined $track_channel;
@@ -186,37 +210,87 @@ sub process {
             $analysis .= "Track $i. Channel: $track_channel\n";
 
             # Declare the notes to inspect
-            my $text = '';
+            my $note_text = '';
+            my $dura_text = '';
 
-            my @group;
-            my $last;
+            my @note_group;
+            my $note_last;
 
             # Accumulate the notes
             for my $event ( @events ) {
                 # Transliterate MIDI note numbers to alpha-code
-                ( my $str = $event->[3] ) =~ tr/0-9/a-j/;
-                $text .= "$str ";
+                ( my $str = $event->[2] ) =~ tr/0-9/a-j/;
+                $dura_text .= "$str ";
+                ( $str = $event->[4] ) =~ tr/0-9/a-j/;
+                $note_text .= "$str ";
 
-                if (@group == $self->ngram_size) {
-                    my $group = join ' ', @group;
-                    $self->net->{ $last . '-' . $group }++ if $last;
-                    $last = $group;
-                    @group = ();
+                if (@note_group == $self->ngram_size) {
+                    my $group = join ' ', @note_group;
+                    $self->net->{ $note_last . '-' . $group }++ if $note_last;
+                    $note_last = $group;
+                    @note_group = ();
                 }
-                push @group, $event->[3];
+                push @note_group, $event->[4];
             }
 
             # Parse the note text into ngrams
-            my $ngram  = Lingua::EN::Ngram->new( text => $text );
-            my $phrase = $ngram->ngram( $self->ngram_size );
+            my $dura_ngram = Lingua::EN::Ngram->new( text => $dura_text );
+            my $dura_phrase = $dura_ngram->ngram( $self->ngram_size );
+            my $note_ngram = Lingua::EN::Ngram->new( text => $note_text );
+            my $note_phrase = $note_ngram->ngram( $self->ngram_size );
 
             # Counter for the ngrams seen
             my $j = 0;
 
+            $analysis .= "\tDurations:\n";
+
             # Display the ngrams in order of their repetition amount
-            for my $p ( sort { $phrase->{$b} <=> $phrase->{$a} || $a cmp $b } keys %$phrase ) {
+            for my $p ( sort { $dura_phrase->{$b} <=> $dura_phrase->{$a} || $a cmp $b } keys %$dura_phrase ) {
                 # Skip single occurance phrases if requested
-                next if !$self->single_phrases && $phrase->{$p} == 1;
+                next if !$self->single_phrases && $dura_phrase->{$p} == 1;
+
+                # Don't allow phrases that are not the right size
+                my @items = grep { $_ } split /\s+/, $p;
+                next unless @items == $self->ngram_size;
+
+                $j++;
+
+                # End if a max is set and we are past the maximum
+                last if $self->max_phrases > 0 && $j > $self->max_phrases;
+
+                # Transliterate our letter code back to MIDI note numbers
+                ( my $num = $p ) =~ tr/a-j/0-9/;
+
+                # Convert MIDI numbers to named durations.
+                my $text = _dura_convert($num);
+
+                $analysis .= sprintf "\t%d\t%d\t%s (%s)\n",
+                    $j, $dura_phrase->{$p}, $num, $text;
+
+                # Save the number of times the phrase is repeated
+                $self->dura->{$track_channel}{$text} += $dura_phrase->{$p};
+            }
+
+            unless (@{ $self->durations }) {
+                # Build the known durations set
+                for my $channel (keys %{ $self->dura }) {
+                    for my $duras (keys %{ $self->dura->{$channel} }) {
+                        my @duras = split / /, $duras;
+                        push @{ $self->_dura_list->{$channel} }, @duras;
+                    }
+                    $self->_dura_list->{$channel} = [ uniq @{ $self->_dura_list->{$channel} } ];
+                }
+            }
+
+            # Reset counter for the ngrams seen
+            $j = 0;
+
+            $analysis .= "\tNotes:\n";
+
+            # Display the ngrams in order of their repetition amount
+            for my $p ( sort { $note_phrase->{$b} <=> $note_phrase->{$a} || $a cmp $b } keys %$note_phrase ) {
+                # Skip single occurance phrases if requested
+                next if !$self->single_phrases && $note_phrase->{$p} == 1;
 
                 # Don't allow phrases that are not the right size
                 my @items = grep { $_ } split /\s+/, $p;
@@ -231,12 +305,13 @@ sub process {
                 ( my $num = $p ) =~ tr/a-j/0-9/;
 
                 # Convert MIDI numbers to named notes.
-                my $text = _convert($num);
+                my $text = _note_convert($num);
 
-                $analysis .= sprintf "\t%d\t%d\t%s %s\n", $j, $phrase->{$p}, $num, $text;
+                $analysis .= sprintf "\t%d\t%d\t%s (%s)\n",
+                    $j, $note_phrase->{$p}, $num, $text;
 
                 # Save the number of times the phrase is repeated
-                $self->notes->{$track_channel}{$num} += $phrase->{$p};
+                $self->notes->{$track_channel}{$num} += $note_phrase->{$p};
             }
 
             $analysis .= $self->_gestalt_analysis( \@events )
@@ -295,14 +370,15 @@ sub populate {
                     );
 
                     # Convert MIDI numbers to named notes.
-                    my $text = _convert($choice);
+                    my $note_text = _note_convert($choice);
 
-                    $playback .= "\t$n\t$channel\t$choice $text\n";
+                    $playback .= "\t$n\t$channel\t$choice $note_text\n";
 
                     # Add each chosen note to the score
                     for my $note ( split /\s+/, $choice ) {
-                        # XXX This is not sophisticated at all
-                        my $duration = $self->durations->[ int rand @{ $self->durations } ];
+                        my $duration = @{ $self->durations }
+                            ? $self->durations->[ int rand @{ $self->durations } ]
+                            : $self->_dura_list->{$channel}[ int rand @{ $self->_dura_list->{$channel} } ];
                         $self->score->n( $duration, $note );
                     }
 
@@ -336,9 +412,9 @@ sub populate {
                 $n++;
 
                 # Convert MIDI numbers to named notes.
-                my $text = _convert($phrase);
+                my $note_text = _note_convert($phrase);
 
-                $playback .= "\t$n\t$channel\t$phrase $text\n";
+                $playback .= "\t$n\t$channel\t$phrase $note_text\n";
 
                 my @phrase = split /\s/, $phrase;
                 push @all, @phrase;
@@ -357,8 +433,9 @@ sub populate {
                         $self->score->r( $self->pause_duration );
                     }
                     else {
-                        # XXX This is not sophisticated at all
-                        my $duration = $self->durations->[ int rand @{ $self->durations } ];
+                        my $duration = @{ $self->durations }
+                            ? $self->durations->[ int rand @{ $self->durations } ]
+                            : $self->_dura_list->{$channel}[ int rand @{ $self->_dura_list->{$channel} } ];
                         $self->score->n( $duration, $note );
                     }
                 }
@@ -384,20 +461,45 @@ sub _random_patch {
     return $self->patches->[ int rand @{ $self->patches } ];
 }
 
-# Convert MIDI numbers to named notes.
-sub _convert {
+# Convert MIDI numbers to named durations.
+sub _dura_convert {
     my $string = shift;
 
-    my $text = '( ';
+    my @text;
+
+    my $match = 0;
+
+    for my $n ( split /\s+/, $string ) {
+        my $dura = $n / 96 / 10;
+
+        for my $key (keys %MIDI::Simple::Length) {
+            if (sprintf('%.4f', $MIDI::Simple::Length{$key}) eq sprintf('%.4f', $dura)) {
+                $match++;
+                $dura = $key;
+                last;
+            }
+        }
+
+        push @text, $match ? $dura : 'd' . $n;
+
+        $match = 0;
+    }
+
+    return join ' ', @text;
+}
+
+# Convert MIDI numbers to named notes.
+sub _note_convert {
+    my $string = shift;
+
+    my @text;
 
     for my $n ( split /\s+/, $string ) {
         my $note = Music::Note->new( $n, 'midinum' );
-        $text .= $note->format('midi') . ' ';
+        push @text, $note->format('midi');
     }
 
-    $text .= ')';
-
-    return $text;
+    return join ' ', @text;
 }
 
 sub _is_integer0 {
@@ -434,7 +536,7 @@ MIDI::Ngram - Find the top repeated note phrases of MIDI files
 
 =head1 VERSION
 
-version 0.1106
+version 0.1200
 
 =head1 SYNOPSIS
 
@@ -459,13 +561,19 @@ version 0.1106
 
   # Analyze multitrack tunes
   $mng = MIDI::Ngram->new(
-    in_file     => [ '/multi/channel/tune1.mid', '/multi/channel/tune2.mid' ],
-    one_channel => 1,
+    in_file => [ '/multi/channel/tune1.mid', '/multi/channel/tune2.mid' ],
   );
 
   $mng->process;
 
-  # Dump out the phrases in order
+  # Dump out the channel 0 duration phrases in order
+  print Dumper [
+    map { "$_ => " . $mng->dura->{0}{$_} }
+      sort { $mng->dura->{0}{$a} <=> $mng->dura->{0}{$b} }
+        keys %{ $mng->dura->{0} }
+  ];
+
+  # Dump out the channel 0 note phrases in order
   print Dumper [
     map { "$_ => " . $mng->notes->{0}{$_} }
       sort { $mng->notes->{0}{$a} <=> $mng->notes->{0}{$b} }
@@ -509,9 +617,12 @@ Default: C<100>
 
 =head2 durations
 
-The note durations to choose from (at random).
+The optional MIDI note durations to choose from (at random).
 
-Default: C<[hn qn en]>
+Default: C<[]> (i.e. use the computed B<dura> phrases instead)
+
+Using a setting of C<['qn']> allows you to evenly inspect the phrases
+during audio playback.
 
 =head2 patches
 
@@ -590,8 +701,13 @@ the B<populate> method.
 
 =head2 notes
 
-The hash-reference bucket of ngrams.  Constructed by the B<process>
-method.
+The hash-reference bucket of pitch ngrams.  Constructed by the
+B<process> method.
+
+=head2 dura
+
+The hash-reference bucket of duration ngrams.  Constructed by the
+B<process> method.
 
 =head2 net
 
@@ -624,10 +740,6 @@ Add notes to the MIDI score and return the playback notes.
 
 Write out the MIDI file.
 
-=head1 TO DO
-
-Preserve note durations instead of random assignment.
-
 =head1 SEE ALSO
 
 L<Moo>
